/* 
PTV04 The Doof Machine - Instructables Edition
Peter Polmeartagami - PTV ArtSpace 2023
Create more music machines! Create more music!

Inspired by Gavin Davies' work https://gavd.co.uk/2022/03/arduino-based-drum-machine/
*/

#include <MozziGuts.h>
#include <Sample.h>
#include "sfx/bom01.h"
#include "sfx/bom02.h"
#include "sfx/bom03.h"
#include "sfx/chi04.h"
#include "sfx/chi01.h"
#include "sfx/chi02.h"
#include "sfx/chi03.h"
#include "sfx/bom04.h"

#include <EventDelay.h>
#include <mozzi_rand.h>

#define CONTROL_RATE 128
#define CELLS_PER_SAMPLE 2048

Sample<CELLS_PER_SAMPLE, AUDIO_RATE> aBom01(BOM01_DATA);
Sample<CELLS_PER_SAMPLE, AUDIO_RATE> aBom02(BOM02_DATA);
Sample<CELLS_PER_SAMPLE, AUDIO_RATE> aBom03(BOM03_DATA);
Sample<CELLS_PER_SAMPLE, AUDIO_RATE> aChi04(CHI04_DATA);
Sample<CELLS_PER_SAMPLE, AUDIO_RATE> aChi01(CHI01_DATA);
Sample<CELLS_PER_SAMPLE, AUDIO_RATE> aChi02(CHI02_DATA);
Sample<CELLS_PER_SAMPLE, AUDIO_RATE> aChi03(CHI03_DATA);
Sample<CELLS_PER_SAMPLE, AUDIO_RATE> aBom04(BOM04_DATA);

Sample<CELLS_PER_SAMPLE, AUDIO_RATE>* sample1 = &aBom01;
Sample<CELLS_PER_SAMPLE, AUDIO_RATE>* sample2 = &aChi01;

EventDelay kTriggerDelay;
const byte MAX_STEP_COUNT = 16;
byte cursor1 = 0;
byte cursor2 = 0;

void setup() {
  pinMode(2, INPUT_PULLUP); // ON/OFF switch if you want to add one.
  pinMode(3, INPUT_PULLUP);
  pinMode(4, INPUT_PULLUP);
  pinMode(5, INPUT_PULLUP);
  pinMode(6, INPUT_PULLUP);
  pinMode(7, INPUT_PULLUP);
  pinMode(8, INPUT_PULLUP);
  pinMode(9, OUTPUT);
  pinMode(10, INPUT_PULLUP);
  pinMode(11, INPUT_PULLUP);
  startMozzi();

  aBom01.setFreq((float)BOM01_SAMPLERATE / (float)CELLS_PER_SAMPLE);
  aBom02.setFreq((float)BOM02_SAMPLERATE / (float)CELLS_PER_SAMPLE);
  aBom03.setFreq((float)BOM03_SAMPLERATE / (float)CELLS_PER_SAMPLE);
  aChi04.setFreq((float)CHI04_SAMPLERATE / (float)CELLS_PER_SAMPLE);
  aChi01.setFreq((float)CHI01_SAMPLERATE / (float)CELLS_PER_SAMPLE);
  aChi02.setFreq((float)CHI02_SAMPLERATE / (float)CELLS_PER_SAMPLE);
  aChi03.setFreq((float)CHI03_SAMPLERATE / (float)CELLS_PER_SAMPLE);
  aBom04.setFreq((float)BOM04_SAMPLERATE / (float)CELLS_PER_SAMPLE);
  
  kTriggerDelay.set(10);
}

bool shouldPlay(byte steps, byte beats, byte i) {
  float divisor = ((float)beats / (float)steps);
  bool shouldPlay;
  if (i == 0) {
    shouldPlay = (beats != 0);
  } else {
    byte score = i * divisor;
    byte prevScore = (i - 1) * divisor;
    shouldPlay = (score > prevScore);
  }
  return shouldPlay;
}

void updateControl() {  // ON/OFF switch if you want to add one.
  if (digitalRead(2) == LOW) {
    return;
  }

  if (digitalRead(3) == LOW) {
    sample1 = &aBom01;
  } else if (digitalRead(4) == LOW) {
    sample1 = &aBom02;
  } else if (digitalRead(5) == LOW) {
    sample1 = &aBom03;
  } else if (digitalRead(6) == LOW) {
    sample1 = &aChi04;
  } else {
    // You can assign different samples or set sample1 to nullptr if you want no sound.
  }

  if (digitalRead(7) == LOW) {
    sample2 = &aChi01;
  } else if (digitalRead(8) == LOW) {
    sample2 = &aChi02;
  } else if (digitalRead(10) == LOW) {
    sample2 = &aChi03;
  } else if (digitalRead(11) == LOW) {
    sample2 = &aBom04;
  } else {
    // You can assign different samples or set sample2 to nullptr if you want no sound.
  }

  byte steps1 = (byte)map(mozziAnalogRead(4), 0, 1023, 1, MAX_STEP_COUNT);
  byte beats1 = (byte)map(mozziAnalogRead(1), 0, 1023, 0, steps1);
  byte steps2 = (byte)map(mozziAnalogRead(5), 0, 1023, 1, MAX_STEP_COUNT);
  byte beats2 = (byte)map(mozziAnalogRead(2), 0, 1023, 0, steps2);

  if (kTriggerDelay.ready()) {
    if (cursor1 >= steps1) {
      cursor1 = 0;
    }
    if (cursor2 >= steps2) {
      cursor2 = 0;
    }

    if (shouldPlay(steps1, beats1, cursor1)) {
      (*sample1).start();
    }
    if (shouldPlay(steps2, beats2, cursor2)) {
      (*sample2).start();
    }

    cursor1++;
    cursor2++;

    kTriggerDelay.start(map(mozziAnalogRead(3), 0, 1023, 300, 50));
  }
}

int updateAudio() {
  int asig = (int)(((long)(*sample1).next() + (*sample2).next()) * 127);
  if (asig > 127) {
    asig = 127;
  } else if (asig < -128) {
    asig = -128;
  }
  return asig;
}

void loop() {
  audioHook();
}
